/**************************************************************************\
| PIC2RLE v1.1 -- Title screen compressor                                  |
|                                                                          |
| David Phillips <electrum@tfs.net>                                        |
| started:     08/19/98                                                    |
| last update: 11/05/98                                                    |
|                                                                          |
| The format for RLE encoded pictures is simple.  Single bytes are encoded |
| as themselves.  Runs of 3 or more bytes are encoded as $91, followed by  |
| the byte value, ended with the number of bytes.  The single value of $91 |
| is encoded as a run of one.  The maximum length of a run is 255, because |
| runs are encoded as a single byte.  This is very close to PCX encoding.  |
\**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>

//======== TYPEDEFS/STRUCTS ===============================================
typedef struct
{
  char  sig[8],         // "**TI86**"
        ext_sig[3],     // EOF, CR, NULL byte (1a, 0a, 00)
        comment[42];    // padded with NULL bytes to len of 42
  short file_len;       // file length minus 0x39
} FileHeader;

typedef struct
{
  short data_ptr,       // points to var_len2 at end of entry
        var_len;        // length of variable
  char  type,           // variable type
        name_len,       // variable name length
        name[8];        // variable name
  short var_len2;       // exactly the same as var_len
} FileEntry;

typedef struct
{
  short len;            // length of the program
  char  token[2];       // for a compiled asm prgm, (8e, 28)
} VarPrgm;

typedef struct
{
  short len;            // length of the string
} VarString;

//======== GLOBAL VARIABLES ===============================================
char pic[1024];

//======== FUNCTIONS ======================================================
void read_pic(char *fname)
{
  FILE        *fp;
  FileHeader  header;
  FileEntry   entry;
  VarPrgm     prgm;

  fp = fopen(fname, "rb");
  if (fp == NULL)
  {
    printf("Cannon open file %s -- make sure it is in\n", fname);
    printf("the same directory (folder) that you ran PIC2RLE from.\n");
    exit(1);
  }
  else
  {
    fread(&header, sizeof(FileHeader), 1, fp);
    fread(&entry, sizeof(FileEntry), 1, fp);
    fread(&prgm, sizeof(VarPrgm), 1, fp);
      
    fread(&pic, 1024, 1, fp);
    
    fclose(fp);
    if (errno != 0)
    {
      printf("Error reading %s -- file is corrupt!\n", fname);
      exit(1);
    }
  }
}

void rle_pic(char *fname)
{
  FILE *fp;
  int  x,
       size,
       run;
  char last;
  
  fp = fopen(fname, "w");
  if (fp == NULL)
  {
    printf("\nError opening %s.  PIC not written!\n", fname);
    exit(1);
  }
  
  fprintf(fp, "; compressed picture made with RLE2PIC\n");
  fprintf(fp, "; RLE2PIC by David Phillips <electrum@tfs.net>\n\n");
  
  size = 0;
  run = 1;
  last = pic[0];
  for(x = 1; x < 1025; x++)
  {
    if ((x != 1024) && (pic[x] == last) && (run < 255))   // will early out
      run++;
    else
    {
      if ((run > 2) || (last == 0x91))
      {
        fput_num(fp, 0x91);
        fput_num(fp, last); 
        fput_num(fp, run);
        size += 3;
      }
      else
      {
        fput_num(fp, last);
        size++;
        if (run == 2)
        {
          fput_num(fp, last);
          size++;
        }
      }
      if (x != 1024)
      {
        run = 1;
        last = pic[x];
      }
    }
  }
  
  x = (int)(((float)size / 1024) * 100);
  fprintf(fp, "\n\n; 1024 bytes compressed to %i", size);
  fprintf(fp, " -- %i\%% of original.\n", x);
  fclose(fp);
  printf("RLE PIC was written as %s.\n", fname);
  printf("1024 bytes compressed to %i -- %i\%% of original.\n", size, x);
}

void fput_num(FILE *fp, char value)
{
  static num = 0;

  if (num % 12 == 0)
    fprintf(fp, "\n .db $%02x", value);
  else
      fprintf(fp, ",$%02x", value);
  num++;
}

//======== MAIN() =========================================================
void main(int argc, char *argv[])
{
  printf("\nPIC2RLE -- by David Phillips <electrum@tfs.net>\n\n");
  
  if (argc == 1)
  {
    printf("Usage:  PIC2RLE <prgmfile> [outfile]\n\n");
    printf("  Prgmfile is a compiled asm program with 1024 bytes of\n");
    printf("  data to compress using RLE compression.  If outfile isn't\n");
    printf("  specified, compressed picture is written to PIC_OUT.ASM.\n");
    exit(1);
  }
  
  read_pic(argv[1]);
  
  if (argc == 2)
    rle_pic("pic_out.asm");
  else
      rle_pic(argv[2]);
}
